/* chanserv.h - Channel service bot
 * Copyright 2000-2001 srvx Development Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: chanserv.h,v 1.25.2.5 2002/01/22 21:48:11 zoot Exp $
 */

#ifndef _chanserv_h
#define _chanserv_h

enum userLevel
{
    ulNone = 0,
    ulPeon,
    ulOp,
    ulMaster,
    ulCoowner,
    ulOwner,
    ulHelper
};

#define USER_LEVEL_HIGHEST	ulOwner
#define USER_LEVEL_LOWEST	ulPeon

enum opt
{
    optProtect = 0,
    optStrictOp,
    optAutoOp,
    optEnfModes,
    optEnfTopic,
    optPubCmd,
    optGreet,
    optToys,
    optMax
};

#define OPTION_STRING_SIZE	(optMax + 2)
#define FLAGS_STRING_LENGTH	5

/* Flags are stored in base64, so a length of 5 gives us
   5 * log2(64) = 30 bits to store on/off options.*/
#define CHANNEL_NODELETE	0x00000001 /* (1 << 0) */
#define CHANNEL_SUSPENDED	0x00000002 /* (1 << 1) */

#define CHANNEL_INFO_LINES	0x00000004 /* (1 << 2) */

#define CHANNEL_VOICE_ALL    	0x00000008 /* (1 << 3) */
#define CHANNEL_GREET_ALL    	0x00000010 /* (1 << 4) */
#define CHANNEL_GREET_USERS     0x00000080 /* (1 << 7) */

#define CHANNEL_DYNAMIC_LIMIT	0x00000020 /* (1 << 5) */

#define CHANNEL_TOPIC_SNARF	0x00000040 /* (1 << 6) */
/* WARNING: Flags with values over 0x20000000 or (1 << 29)
   will not work. */

/* WARNING: This define *must* be updated when new flags are
   added. Set it to the total number of flags. */
#define CHANNEL_FLAGS_SIZE	8

#define CHANNEL_DEFAULT_FLAGS	(CHANNEL_INFO_LINES)
#define CHANNEL_DEFAULT_OPTIONS	"lmoooanp"

#define IsProtected(x)		((x)->flags & CHANNEL_NODELETE)
#define IsSuspended(x)		((x)->flags & CHANNEL_SUSPENDED)

struct chanData
{
    struct chanNode	*channel;

    time_t		registered;
    time_t		visited;
    time_t 		limitAdjusted;

    char		options[OPTION_STRING_SIZE];
    char		topic[TOPICLEN + 1];
    char		key[KEYLEN + 1];
    char		*greeting;
    char		*registrar;

    unsigned int	limit;
    unsigned int	flags : CHANNEL_FLAGS_SIZE;
    long		mode_on;
    long		mode_off;

    struct userData	*users;
    struct banData	*bans;

    unsigned int	userCount[USER_LEVEL_HIGHEST + 1];
    unsigned int	banCount;
    unsigned int	max;

    struct suspended	*suspended;

    struct chanData	*prev;
    struct chanData	*next;
};

struct userData
{
    struct handle_info	*handle;

    char		*info;
    time_t 		seen;
    int			present : 1;

    int			level;
    enum userLevel	access;

    struct userData	*prev;
    struct userData	*next;
};

struct banData
{
    char		mask[NICKLEN + USERLEN + HOSTLEN + 3];
    char		owner[NICKLEN];
    struct chanData     *channel;

    time_t		set;
    time_t		triggered;
    unsigned long      	duration;

    char		*reason;

    struct banData	*prev;
    struct banData	*next;
};

struct chanList
{
    struct chanData	*channel;
    struct userData	*user;

    struct chanList	*next;
    struct chanList	*prev;
};

struct suspended
{
    struct chanData	*cData;

    char		*name;
    char		*suspender;
};

void init_chanserv(const char *nick);

#endif
